@@ -102,7 +102,17 @@ def execute_parted(args):
         raise PartedNotInstalledException()
     parted.wait()
     if parted.get_returncode() != 0:
-        log.debug("parted returned non-zero exit code: {}".format(parted.get_returncode()))
+        log.debug("parted returned non-zero exit code: {}, stderr and stdout to follow".format(parted.get_returncode()))
+        log.debug(parted.get_stderr())
+        log.debug(parted.get_stdout())
+        if "device-mapper: create ioctl" in parted.get_stderr():
+            # this happens sometimes on redhat-7
+            # at first we added a retry, but the repeating execution printed:
+            # You requested a partition from 65536B to 999934464B (sectors 128..1952997).
+            # The closest location we can manage is 65024B to 65024B (sectors 127..127).
+            # meaning the first execution suceeded to create the partition
+            # so now we're just ignore the return code in case we see this message
+            return parted.get_stdout()
         if "WARNING" in parted.get_stdout():
             # don't know what's the error code in this case, and failed to re-create it
             return parted.get_stdout()
@@ -261,12 +271,13 @@ class Disk(MatchingPartedMixin, Retryable, object):
         from infi.execute import execute
         execute(["partprobe", format(self._device_access_path)]).wait()
 
-    @retry_func(WaitAndRetryStrategy(max_retries=5, wait=5))
+    @retry_func(WaitAndRetryStrategy(max_retries=120, wait=5))
     def _execute_mkfs(self, filesystem_name, partition_access_path):
         from infi.execute import execute
         log.info("executing mkfs.{} for {}".format(filesystem_name, partition_access_path))
         mkfs = execute(["mkfs.{}".format(filesystem_name), "-F", partition_access_path])
         if mkfs.get_returncode() != 0:
+            logger.debug("mkfs failed (): {} {}".format(mkfs.get_returncode(), mkfs.get_stdout(), mkfs.get_stderr()))
             raise RuntimeError(mkfs.get_stderr())
         log.info("filesystem formatted")
 
@@ -322,6 +333,16 @@ class Partition(object):
         from infi.execute import execute
         execute(["partprobe", format(self._disk_block_access_path)]).wait()
 
+    def get_filesystem_name(self):
+        return self._filesystem or self.get_filesystem_name_from_blkid()
+
+    def get_filesystem_name_from_blkid(self):
+        from infi.execute import execute_assert_success
+        from re import search
+        output = execute_assert_success(["blkid", self.get_access_path()]).get_stdout()
+        return search(r'TYPE="([^\"]+)*"', output).group(1)
+
+
 class MBRPartition(Partition):
     def __init__(self, disk_block_access_path, number, partition_type, start, end, size, filesystem):
         super(MBRPartition, self).__init__(disk_block_access_path, number, start, end, size)
@@ -338,9 +359,6 @@ class MBRPartition(Partition):
         prefix = get_multipath_prefix(self._disk_block_access_path) if 'mapper' in self._disk_block_access_path else ''
         return "{}{}{}".format(self._disk_block_access_path, prefix, self._number)
 
-    def get_filesystem_name(self):
-        return self._filesystem or None
-
     def resize(self, size_in_bytes):
         raise NotImplementedError()
 
@@ -373,9 +391,6 @@ class GUIDPartition(Partition):
         prefix = get_multipath_prefix(self._disk_block_access_path) if 'mapper' in self._disk_block_access_path else ''
         return "{}{}{}".format(self._disk_block_access_path, prefix, self._number)
 
-    def get_filesystem_name(self):
-        return self._filesystem or None
-
     @classmethod
     def from_parted_machine_parsable_line(cls, disk_device_path, line):
         number, start, end, size, filesystem, name, flags = line.strip(';').split(':')
